@@ -14,8 +14,8 @@ from webargs.core import ValidationError
 __version__ = importlib.metadata.version("webargs")
 __parsed_version__ = Version(__version__)
 __version_info__: tuple[int, int, int] | tuple[int, int, int, str, int] = (
-    __parsed_version__.release
-)  # type: ignore[assignment]
+    __parsed_version__.release  # type: ignore[assignment]
+)
 if __parsed_version__.pre:
     __version_info__ += __parsed_version__.pre  # type: ignore[assignment]
 __all__ = ("ValidationError", "fields", "missing", "validate")
